
#include <Wire.h>
#include <OneWire.h>
#include <TimerOne.h>
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27, 20, 4);
//#include <LiquidCrystal.h>
//LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

#define BUTTON_PIN 12
#define T_PRESS 1800
#define T_CLICK 2500
#define T_PRESS_LONG 5000
#define BUTTON_CLICK  1
#define BUTTON_DOUBLE 2
#define BUTTON_LONG   0
#define BUTTON_VERY_LONG 3

//#define BEEP_PIN 13
#define BEEP_PIN 2
//#define VALVE_PIN 2
#define VALVE_PIN 13
#define DS_PIN 11
#define BEEP_FREQ 250
#define BEEP_ON 1000
#define BEEP_OFF 1000

#define POWER_MODE 1 // 1 - паразитное, 0 - внешнее
#define DS_PERIOD 4000 // 1 сек
byte cntCRC = 0;

#define tempAlarmMin 70.0
#define tempAlarm1 95.0
#define tempAlarm2 99.9
#define tempRazgMin 30.0
#define tempRazgMax 70.0
#define VALVE_PERIOD 1000  // 0.25 сек

byte jobState = 0;
#define BRAGAorSS 0
#define BragaConfirmation 1
#define SSConfirmation 2
#define Braga 3
#define Razgon 4
#define RazgonEnd 5
#define NaSebya 6
#define NaSebyaEnd 7
#define ShimG 8
#define ShimGConfirmation 9
#define Golovy 10
#define ShimT 11
#define ShimTConfirmation 12
#define Telo 13
#define AutoConfirmation 14
#define Auto 15
#define ShimA 16
#define ShimAConfirmation 17
#define Tauto 18
#define TautoConfirmation 19
#define StartStop 20
#define OtborEnd 21
#define ShimSS 22
#define ShimSSConfirmation 23
#define TautoSS 24
#define TautoSSConfirmation 25

OneWire sensDs (DS_PIN);

// обслуживание сенсорной кнопки
boolean startButton = false;
byte stateButton = 0;
int counterButton = 0;
int counterButtonDelay = 700;

class Button {
  public:
    boolean buttonClick = false;
    byte buttonType; // 0 - Long type, 1 - Type, 2 -Double type,  3 - VeryLong
    int lastStateButton = 0;

    void scanState();

  private:
    int buttonCode;
    boolean buttonPress = false;
};

Button button1;

int buttonIn; // какой тип выбрать?
int dsCount;
int copydsCount;
boolean dsFlagReady;
byte dsBuf[9];
float temperature;
float tempRazg = 30.0;
float tempAlarm;
float tempAuto;
float tempAutoTmp;
// float newtempAuto;
float hist = 0.125;

// для клапана
byte valveOn = 0;
byte valveOnTmp = 0;
byte countOn = 0;
byte countOff = 0;
byte valveOff = 0;
byte valveOffTmp = 0;
unsigned int valveCount;
#define valveOnHeadMax 12
#define valveOnHeadMin 2
#define valveOnBodyMax 14
#define valveOnBodyMin 2
byte valveOnHead = 4;
byte valveOffHead = 56; // 1 сек открыто из 15
byte valveOnBody = 10;
byte valveOffBody = 6;  // 2.5 сек открыто из 4
byte modeValve = LOW; // LOW - клапан закрыт, HIGH - открыт.
//byte vRed = 0; // 0 - редактируем valveOn Головы, 1 - valveOn Тело
// 2 - 10 valveOff, 3 - 1 valveOff

//boolean k; // для клавиатуры

// для бибикалки
byte beepOut;
unsigned int beepCount;
boolean beepFlagOn;

// для таймера
unsigned int timerCount; // счётчик до 1 сек.
byte timerMin;
byte timerSec;
boolean secondFlagOn;
#define timerNaSebya 25 // минут

// для старт-стопа
#define timerStop  8 // минут

//byte addrDS[8]={0x28, 0xFF, 0x83, 0x73, 0xB5, 0x16, 0x05, 0x1D}; // - номер датчика температуры - Китай
byte addrDS[8] = {0x28, 0xF8, 0x2C, 0x63, 0x1E, 0x19, 0x1, 0xB2}; // СИВ

void setup() {
  lcd.init();            // ------------------------ i2c или нет
  lcd.backlight();
  //  lcd.begin(16, 2);
  pinMode (BUTTON_PIN, INPUT);
//  Serial.begin(115200);
  beepOut = HIGH;
  pinMode(BEEP_PIN, OUTPUT);
  digitalWrite(BEEP_PIN, beepOut);
  beepCount = 0;
  Timer1.initialize(BEEP_FREQ);
  Timer1.attachInterrupt(beepOutput);
  dsCount = 0;
  dsFlagReady = true;
  tempAlarm = tempAlarmMin; // изменяется в ходе выполнения
  beepFlagOn = false;
  jobState = 0;
  menuBRAGAorSS ();
}
//=======================================================================
void loop() {
  choiceJobState ();
  temperature_measurement ();
}
//============= конец loop ==============================================

void choiceJobState () {
  switch (jobState) {
    case BRAGAorSS:
      button1.scanState();
      if (buttonLong ()) {
        button1.buttonClick = false; jobState = 1;
        lcd.setCursor(0, 0); lcd.print("     BRAGA      ");
        lcd.setCursor(0, 1); lcd.print("Y-Long  N-Double");
      }
      if (buttonDouble ()) {
        button1.buttonClick = false; jobState = 2;
        lcd.setCursor(0, 0); lcd.print("       SS       ");
        lcd.setCursor(0, 1); lcd.print("Y-Long  N-Double");
      }
      break;
    //========================================================================
    case BragaConfirmation:
      button1.scanState();
      if (buttonDouble ()) {
        button1.buttonClick = false; jobState = BRAGAorSS;
        menuBRAGAorSS ();
      }
      if (buttonLong ()) {
        button1.buttonClick = false; jobState = Braga;
        lcd.clear();
        lcd.setCursor(0, 0); lcd.print("Temp:     "); lcd.print(temperature);
        lcd.setCursor(0, 1); lcd.print("Water on: "); lcd.print(tempAlarm);
      }
      break;
    //==========================================================================
    case SSConfirmation:
      button1.scanState();
      if (buttonDouble ()) {
        jobState = BRAGAorSS; button1.buttonClick = false;
        menuBRAGAorSS ();
      }
      if (buttonLong ()) {
        jobState = Razgon ; button1.buttonClick = false;
        menuRazgon ();
      }
      break;
    //==========================================================================
    case Braga:
      lcd.setCursor(10, 0); lcd.print(temperature);
      button1.scanState();
      if ((buttonOne ()) & (tempAlarm == tempAlarmMin)) {
        tempAlarm = tempAlarm1;
        lcd.setCursor(0, 1); lcd.print("TempStop: "); lcd.print(tempAlarm);
        button1.buttonClick = false;
      }
      if ((buttonOne ()) & (tempAlarm == tempAlarm1)) {
        tempAlarm = tempAlarm2;
        lcd.setCursor(0, 1); lcd.print("StopBeep: "); lcd.print(tempAlarm);
        button1.buttonClick = false;
      }
      if ((buttonOne ()) & (tempAlarm == tempAlarm2)) {
        tempAlarm = tempAlarmMin;
        lcd.setCursor(0, 1); lcd.print("Water on: "); lcd.print(tempAlarm);
        button1.buttonClick = false;
      }
      lcd.setCursor(10, 1); lcd.print(tempAlarm);
      if (temperature < tempAlarm) {
        beepFlagOn = false; digitalWrite(BEEP_PIN, HIGH);
      }
      if (temperature >=  tempAlarm) {
        beepFlagOn = true;
      }
      break;
    //========================================================
    case Razgon:
      button1.scanState();
      if (buttonLong ()) {
        jobState = NaSebya ;
        timerMin = timerNaSebya; timerSec = 0;
        menuNaSebya ();
        secondFlagOn = false;
        timerCount = 0;
      }
      if (buttonOne () & (tempRazg >= 35.0)) {
        tempRazg = tempRazg - 5.0;
        lcd.setCursor(15, 1); lcd.print(tempRazg);
      }
      if (buttonDouble () & (tempRazg <= 65.0)) {
        tempRazg = tempRazg + 5.0;
        lcd.setCursor(15, 1); lcd.print(tempRazg);
      }
      button1.buttonClick = false;

      lcd.setCursor(3, 1); lcd.print(temperature);
      if (temperature >= tempRazg) {
        jobState = RazgonEnd ; menuRazgonEnd ();
        beepFlagOn = true;
      }
      break;
    //==============================================================
    case RazgonEnd:
      button1.scanState();
      if (buttonLong()) {
        jobState = NaSebya ;
        timerMin = timerNaSebya; timerSec = 0;
        beepFlagOn = false; digitalWrite(BEEP_PIN, HIGH);
        menuNaSebya ();
        secondFlagOn = false;
        timerCount = 0;
      }
      button1.buttonClick = false;
      break;
    //=============================================================
    case NaSebya:
      lcd.setCursor(3, 1); lcd.print(temperature);

      if (secondFlagOn == true) {
        secondFlagOn = false;
        if (timerSec == 0) {
          timerSec = 59; timerMin--;
        }
        timerSec--;
        secondFlagOn = false;
      }

      lcd.setCursor(15, 1);
      if (timerMin < 10) {
        lcd.print("0");
      }
      lcd.print(timerMin); lcd.print(F(":"));
      if (timerSec < 10) {
        lcd.print("0");
      }
      lcd.print(timerSec);

      if ((timerMin == 0) & (timerSec == 0)) {
        //      timerFlagOn = false;
        menuNaSebyaEnd ();
        beepFlagOn = true; jobState = NaSebyaEnd;
      }

      button1.scanState();
      if (buttonLong()) {
        //      timerFlagOn = false;
        menuShimG ();
        jobState = ShimG;
      }

      if (buttonOne () & (timerMin >= (timerNaSebya))) {
        timerMin = timerMin - 5 ;
      }

      if (buttonDouble () & (timerMin <= (timerNaSebya + 10))) {
        timerMin = timerMin + 5 ;
      }
      button1.buttonClick = false;
      break;
    //=============================================================
    case NaSebyaEnd:
      button1.scanState();
      if (buttonLong()) {
        beepFlagOn = false; digitalWrite(BEEP_PIN, HIGH);
        valveOn = valveOnHead; valveOff = valveOffHead;
        valveOnTmp = valveOn; valveOffTmp = valveOff;
        menuShimG ();
        jobState = ShimG;
      }
      button1.buttonClick = false;
      break;
    //=============================================================
    case ShimG:
      Shim( valveOnHeadMin, valveOnHeadMax, ShimGConfirmation );
      //    button1.scanState();
      //    k1();
      //    if ((k == true) & (valveOn > valveOnHeadMin)) {
      //      valveOn-- ; valveOff++ ;
      //    }
      //    k2();
      //    if ((k == true) & (valveOn < valveOnHeadMax)) {
      //      valveOn++ ; valveOff-- ;
      //    }
      //    k0();
      //    if (k == true) {
      //      menuShimGConfirmation (); jobState = ShimGConfirmation;
      //    }
      //
      //    lcd.setCursor(7, 1);
      //    if (valveOn < 10) {
      //      lcd.print(F("0"));
      //    }
      //    lcd.print(valveOn);
      //
      //    lcd.setCursor(10, 1);
      //    if (valveOff < 10) {
      //      lcd.print(F("0"));
      //    }
      //    lcd.print(valveOff);
      //
      //    button1.buttonClick = false;
      break;
    //=================================================================
    case ShimGConfirmation:
      ShimConfirmation ( Golovy, ShimG ) ;
      //    button1.scanState();
      //    k0();
      //    if (k == true) {
      //      menuGolovy(); countOn = valveOn; countOff = valveOff;
      //      modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
      //      jobState = Golovy;
      //    }
      //    k2();
      //    if (k == true) {
      //      valveOn = valveOnTmp; valveOff = valveOffTmp;
      //      menuShimG(); jobState = ShimG;
      //    }
      //    button1.buttonClick = false;
      break;
    //================================================================
    case Golovy:
      lcd.setCursor(3, 1); lcd.print(temperature);
      if (valveCount >= VALVE_PERIOD) {
        valveCount = 0;
        if (modeValve == HIGH) {
          countOn--;
          if (countOn == 0) {
            modeValve = LOW; digitalWrite(VALVE_PIN, modeValve);
            countOn = 0; countOff = valveOff;
          }
        }
        else {
          countOff--;
          if (countOff == 0) {
            modeValve = HIGH; digitalWrite(VALVE_PIN, modeValve);
            countOn = valveOn; countOff = 0;
          }
        }
        lcd.setCursor(15, 1);
        if (countOn < 10) {
          lcd.print(F("0"));
        }
        lcd.print(countOn); lcd.print(F("/"));
        if (countOff < 10) {
          lcd.print(F("0"));
        }
        lcd.print(countOff);
      }
      button1.scanState();
      if (buttonLong ()) {
        modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время изменения ШИМ
        valveOn = valveOnBody; valveOff = valveOffBody;
        valveOnTmp = valveOn; valveOffTmp = valveOff;
        menuShimT ();
        jobState = ShimT;
        button1.buttonClick = false;
      }
      if (buttonDouble ()) {
        modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время изменения ШИМ
        valveOnTmp = valveOn; valveOffTmp = valveOff;
        menuShimG ();
        jobState = ShimG;
        button1.buttonClick = false;
      }
      break;
    //====================================================================
    case ShimT:
      Shim( valveOnBodyMin, valveOnBodyMax, ShimTConfirmation );
      //    button1.scanState();
      //    k1();
      //    if ((k == true) & (valveOn > valveOnBodyMin)) {
      //      valveOn-- ; valveOff++ ;
      //    }
      //    k2();
      //    if ((k == true) & (valveOn < valveOnBodyMax)) {
      //      valveOn++ ; valveOff-- ;
      //    }
      //    k0();
      //    if (k == true) {
      //      menuShimTConfirmation (); jobState = ShimTConfirmation;
      //    }
      //
      //    lcd.setCursor(7, 1);
      //    if (valveOn < 10) {
      //      lcd.print(F("0"));
      //    }
      //    lcd.print(valveOn);
      //
      //    lcd.setCursor(10, 1);
      //    if (valveOff < 10) {
      //      lcd.print(F("0"));
      //    }
      //    lcd.print(valveOff);
      //
      //    button1.buttonClick = false;
      break;
    //================================================================
    case ShimTConfirmation:
      ShimConfirmation ( Telo, ShimT ) ;
      //    button1.scanState();
      //    k0();
      //    if (k == true) {
      //      menuTelo(); countOn = valveOn; countOff = valveOff;
      //      modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
      //      jobState = Telo;
      //    }
      //    k2();
      //    if (k == true) {
      //      valveOn = valveOnTmp; valveOff = valveOffTmp;
      //      menuShimT(); jobState = ShimT;
      //    }
      //    button1.buttonClick = false;
      break;
    //===============================================================
    case Telo:
      lcd.setCursor(3, 1); lcd.print(temperature);
      if (valveCount >= VALVE_PERIOD) {
        valveCount = 0;
        if (modeValve == HIGH) {
          countOn--;
          if (countOn == 0) {
            modeValve = LOW; digitalWrite(VALVE_PIN, modeValve);
            countOn = 0; countOff = valveOff;
          }
        }
        else {
          countOff--;
          if (countOff == 0) {
            modeValve = HIGH; digitalWrite(VALVE_PIN, modeValve);
            countOn = valveOn; countOff = 0;
          }
        }
        lcd.setCursor(15, 1);
        if (countOn < 10) {
          lcd.print(F("0"));
        }
        lcd.print(countOn); lcd.print(F("/"));
        if (countOff < 10) {
          lcd.print(F("0"));
        }
        lcd.print(countOff);
      }
      button1.scanState();
      if (buttonDouble ()) {
        modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время изменения ШИМ
        valveOn = valveOnBody; valveOff = valveOffBody;
        valveOnTmp = valveOn; valveOffTmp = valveOff;
        menuShimT ();
        jobState = ShimT;
        button1.buttonClick = false;
      }
      if (buttonLong ()) {
        modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время подтв. AUTO
        //      valveOnTmp = valveOn; valveOffTmp = valveOff;
        menuAutoConfirmation ();
        jobState = AutoConfirmation;
        button1.buttonClick = false;
      }
      break;
    //========================================================================
    case AutoConfirmation:
      button1.scanState();
      if (buttonDouble ()) {
        menuTelo(); countOn = valveOn; countOff = valveOff;
        modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
        jobState = Telo;
        button1.buttonClick = false;
      }
      if (buttonLong ()) {
        menuAuto (); countOn = valveOn; countOff = valveOff;
        modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
        tempAuto = temperature;
        jobState = Auto;
        button1.buttonClick = false;
      }
      break;
    //========================================================================
    case Auto:
      lcd.setCursor(2, 1); lcd.print(temperature);
      lcd.setCursor(15, 1); lcd.print(tempAuto);
      if (valveCount >= VALVE_PERIOD) {
        valveCount = 0;
        if (modeValve == HIGH) {
          countOn--;
          if (countOn == 0) {
            modeValve = LOW; digitalWrite(VALVE_PIN, modeValve);
            countOn = 0; countOff = valveOff;
          }
        }
        else {
          countOff--;
          if (countOff == 0) {
            modeValve = HIGH; digitalWrite(VALVE_PIN, modeValve);
            countOn = valveOn; countOff = 0;
          }
        }
        lcd.setCursor(15, 0);
        if (countOn < 10) {
          lcd.print(F("0"));
        }
        lcd.print(countOn); lcd.print(F("/"));
        if (countOff < 10) {
          lcd.print(F("0"));
        }
        lcd.print(countOff);
      }
      button1.scanState();
      if (buttonDouble ()) {
        modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время изменения ШИМ
        valveOnTmp = valveOn; valveOffTmp = valveOff;
        menuShimA ();
        jobState = ShimA;
        button1.buttonClick = false;
      }
      if (buttonLong ()) {
        modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время подтв. Tauto
        tempAutoTmp = tempAuto;
        menuTauto ();
        jobState = Tauto;
        button1.buttonClick = false;
      }
      if (temperature >= (tempAuto + hist)) {
        modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время START/STOP
        menuStartStop ();
        timerMin = timerStop; timerSec = 0;
        beepFlagOn = false; digitalWrite(BEEP_PIN, HIGH);
        secondFlagOn = false;
        timerCount = 0;
        jobState = StartStop;
      }
      break;
    //==========================================================================
    case ShimA:
      Shim( valveOnHeadMin, valveOnHeadMax, ShimAConfirmation );
      //    button1.scanState();
      //    k1();
      //    if ((k == true) & (valveOn > valveOnBodyMin)) {
      //      valveOn-- ; valveOff++ ;
      //    }
      //    k2();
      //    if ((k == true) & (valveOn < valveOnBodyMax)) {
      //      valveOn++ ; valveOff-- ;
      //    }
      //    k0();
      //    if (k == true) {
      //      menuShimAConfirmation (); jobState = ShimAConfirmation;
      //    }
      //
      //    lcd.setCursor(7, 1);
      //    if (valveOn < 10) {
      //      lcd.print(F("0"));
      //    }
      //    lcd.print(valveOn);
      //
      //    lcd.setCursor(10, 1);
      //    if (valveOff < 10) {
      //      lcd.print(F("0"));
      //    }
      //    lcd.print(valveOff);
      //
      //    button1.buttonClick = false;
      break;
    //=====================================================================
    case ShimAConfirmation:
      ShimConfirmation ( Auto, ShimA ) ;
      //    button1.scanState();
      //    k0();
      //    if (k == true) {
      //      menuAuto(); countOn = valveOn; countOff = valveOff;
      //      modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
      //      jobState = Auto;
      //    }
      //    k2();
      //    if (k == true) {
      //      valveOn = valveOnTmp; valveOff = valveOffTmp;
      //      menuShimA(); jobState = ShimA;
      //    }
      //    button1.buttonClick = false;
      break;
    //=================================================================
    case Tauto:
      button1.scanState();
      if (buttonOne () & (tempAuto > tempAutoTmp)) {
        tempAuto = tempAuto - 0.0625 ;
      }
      if (buttonDouble () & (tempAuto < (tempAutoTmp + 0.25))) {
        tempAuto = tempAuto + 0.0625 ;
      }
      if (buttonLong ()) {
        menuTautoConfirmation (); jobState = TautoConfirmation;
      }
      lcd.setCursor(8, 1); lcd.print(tempAuto);
      button1.buttonClick = false;
      break;
    //=================================================================
    case TautoConfirmation:
      button1.scanState();
      if (buttonLong ()) {
        countOn = valveOn; countOff = valveOff;
        modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
        menuAuto(); jobState = Auto;
      }
      if (buttonDouble ()) {
        tempAuto = tempAutoTmp;
        menuTauto(); jobState = Tauto;
      }
      button1.buttonClick = false;
      break;
    //===================================================================
    case StartStop:
      lcd.setCursor(4, 1); lcd.print(temperature);

      if (secondFlagOn == true) {
        secondFlagOn = false;
        if (timerSec == 0) {
          timerSec = 59; timerMin--;
        }
        timerSec--;
        secondFlagOn = false;
      }

      lcd.setCursor(13, 1);
      if (timerMin < 10) {
        lcd.print("0");
      }
      lcd.print(timerMin); lcd.print(F(":"));
      if (timerSec < 10) {
        lcd.print("0");
      }
      lcd.print(timerSec);

      if ((timerMin == 0) & (timerSec == 0)) {
        //      timerFlagOn = false;
        menuOtborEnd ();
        beepFlagOn = true; jobState = OtborEnd;;
      }

      button1.scanState();
      if (buttonDouble ()) {
        //      modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время изменения ШИМ
        valveOnTmp = valveOn; valveOffTmp = valveOff;
        menuShimA ();
        jobState = ShimSS;
        button1.buttonClick = false;
      }
      if (buttonLong ()) {
        //      modeValve = LOW; digitalWrite(VALVE_PIN, modeValve); // закрыть клапан на время подтв. Tauto
        tempAutoTmp = tempAuto;
        menuTauto ();
        jobState = TautoSS;
        button1.buttonClick = false;
      }
      if (temperature <= tempAuto) {
        countOn = valveOn; countOff = valveOff;
        modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
        menuAuto(); jobState = Auto;
      }
      break;
    //======================================================================
    case ShimSS:
      Shim( valveOnBodyMin, valveOnBodyMax, ShimSSConfirmation );
      //    button1.scanState();
      //    k1();
      //    if ((k == true) & (valveOn > valveOnBodyMin)) {
      //      valveOn-- ; valveOff++ ;
      //    }
      //    k2();
      //    if ((k == true) & (valveOn < valveOnBodyMax)) {
      //      valveOn++ ; valveOff-- ;
      //    }
      //    k0();
      //    if (k == true) {
      //      menuShimAConfirmation (); jobState = ShimSSConfirmation;
      //    }
      //
      //    lcd.setCursor(7, 1);
      //    if (valveOn < 10) {
      //      lcd.print(F("0"));
      //    }
      //    lcd.print(valveOn);
      //
      //    lcd.setCursor(10, 1);
      //    if (valveOff < 10) {
      //      lcd.print(F("0"));
      //    }
      //    lcd.print(valveOff);
      //
      //    button1.buttonClick = false;
      break;
    //=====================================================================
    case ShimSSConfirmation:
      ShimConfirmation ( StartStop, ShimSS );
      //    button1.scanState();
      //    k0();
      //    if (k == true) {
      //      menuStartStop(); jobState = StartStop;
      //    }
      //    k2();
      //    if (k == true) {
      //      valveOn = valveOnTmp; valveOff = valveOffTmp;
      //      menuShimA(); jobState = ShimSS;
      //    }
      //    button1.buttonClick = false;
      break;
    //=====================================================================
    case TautoSS:
      button1.scanState();
      if (buttonOne () & (tempAuto > tempAutoTmp)) {
        tempAuto = tempAuto - 0.0625 ;
      }
      if (buttonDouble () & (tempAuto < (tempAutoTmp + 0.25))) {
        tempAuto = tempAuto + 0.0625 ;
      }
      if (buttonLong ()) {
        menuTautoConfirmation (); jobState = TautoSSConfirmation;
      }
      lcd.setCursor(8, 1); lcd.print(tempAuto);
      button1.buttonClick = false;
      break;
    //=============================================================
    case TautoSSConfirmation:
      button1.scanState();
      if (buttonLong ()) {
        //      countOn = valveOn; countOff = valveOff;
        //      modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
        menuStartStop(); jobState = StartStop;
      }
      if (buttonDouble ()) {
        tempAuto = tempAutoTmp;
        menuTauto(); jobState = TautoSS;
      }
      button1.buttonClick = false;
      break;
    //==============================================================
    case OtborEnd:
      button1.scanState();
      if (buttonLong()) {
        beepFlagOn = false;  digitalWrite(BEEP_PIN, HIGH);
        beepOut = HIGH;
        digitalWrite(BEEP_PIN, beepOut);
      }
      button1.buttonClick = false;
      break;
  }
}

// работа с термодатчиком ====================================================
void temperature_measurement () {
  if (dsFlagReady == true) {
    sensDs.reset();
    sensDs.write(0xCC, POWER_MODE);
    sensDs.write(0x44, POWER_MODE); //запуск измерения
    dsCount = 0; dsFlagReady = false;
  }

  if (copydsCount >= DS_PERIOD) {
    sensDs.reset();
    sensDs.select(addrDS);
    //    sensDs.write(0xCC, POWER_MODE);
    sensDs.write(0xBE, POWER_MODE); //команда чтения
    sensDs.read_bytes(dsBuf, 9);
    if (OneWire::crc8(dsBuf, 8) == dsBuf[8]) {
      temperature = (float)((int)dsBuf[0] | (((int)dsBuf[1]) << 8)) * 0.0625 + 0.3125;
//      Serial.println("ok");
    }
    else {
//      Serial.println("CRC error"); cntCRC++; lcd.setCursor(5, 0); lcd.print("CRC"); lcd.print(cntCRC);
    }
    dsCount = 0; dsFlagReady = true;
  }
}

void beepOutput ()
{
  if (beepFlagOn == true) {
    beepCount++;
    if (beepCount < BEEP_ON) {
      digitalWrite(BEEP_PIN, beepOut);
      beepOut = !beepOut;
    }
    if (beepCount == BEEP_ON) {
      beepOut = HIGH;
      digitalWrite(BEEP_PIN, beepOut);
    }
    if (beepCount >= BEEP_ON + BEEP_OFF) {
      beepCount = 0;
    }
  }
  // работа с термодатчиком
  dsCount++;
  copydsCount = dsCount;

  // работа с клапаном
  valveCount++;

  // работа с сенсорной кнопкой
  if (startButton == true) {
    counterButton++;
  }

  //работа с таймером
  timerCount++;
  if (timerCount == 4000) {
    timerCount = 0;
    secondFlagOn = true;
  }
}

// обработчик кнопки ===================================================
void Button::scanState() {
  buttonIn = digitalRead(BUTTON_PIN);

  // состояние 0 (кнопка не нажата) -------------------
  if (stateButton == 0)
  {
    if ((lastStateButton == 0) & (buttonIn == 1))
    {
      lastStateButton = 1; stateButton = 1; startButton = false; counterButton = 0; startButton = true;
    }
  }

  // состояние 1 (после обнаружения нажатия) ----------
  if (stateButton == 1)
  {
    if ((lastStateButton == 1) & (buttonIn == 0) & (counterButton < T_PRESS))
    {
      lastStateButton = 0; stateButton = 2; startButton = false; counterButton = 0; startButton = true;
    }
    if ((lastStateButton == 1) & (buttonIn == 1) & (counterButton >= T_PRESS))
    {
      if (counterButton >= T_PRESS_LONG)
      {
        stateButton = 5; startButton = false; counterButton = 0;  // очень длительное нажатие
      }
      stateButton = 4; startButton = false; counterButton = 0;  // длительное нажатие
    }
  }

  // состояние 5 (очень длительное нажатие) ------------------------------------------
  if (stateButton == 5)
  {
    if ((lastStateButton == 1) & (buttonIn == 0))
    {
      lastStateButton = 0; buttonType = BUTTON_VERY_LONG; startButton = false; counterButton = 0; stateButton = 0; buttonClick = true ;
    }
  }

  // состояние 4 (длительное нажатие) ------------------------------------------
  if (stateButton == 4)
  {
    if ((lastStateButton == 1) & (buttonIn == 0))
    {
      lastStateButton = 0; buttonType = BUTTON_LONG; startButton = false; counterButton = 0; stateButton = 0; buttonClick = true ;
    }
  }

  // состояние 2 (ждём будет второй клик или нет) ------------------------------
  if (stateButton == 2)
  {
    if ((lastStateButton == 0) & (buttonIn == 1) & (counterButton < T_CLICK))
    {
      lastStateButton = 1; stateButton = 3; startButton = false; counterButton = 0;
    }
    if ((lastStateButton == 0) & (buttonIn == 0) & (counterButton >= T_CLICK))
    {
      buttonClick = true; buttonType = BUTTON_CLICK; startButton = false; counterButton = 0; stateButton = 0;
    }
  }

  // состояние 3 (двойное нажатие кнопки) --------------------------------------
  if (stateButton == 3)
  {
    if ((lastStateButton == 1) & (buttonIn == 0))
    {
      buttonClick = true; lastStateButton = 0; buttonType = BUTTON_DOUBLE; stateButton = 0;
    }
  }
} // закрываем обработчик кнопки ===============================================

boolean buttonVeryLong () {
  if ((button1.buttonClick == true) & (button1.buttonType == BUTTON_VERY_LONG)) {
    return true;
  }
  else {
    return false;
  }
}

boolean buttonLong () {
  if ((button1.buttonClick == true) & (button1.buttonType == BUTTON_LONG)) {
    return true;
  }
  else {
    return false;
  }
}

boolean buttonOne () {
  if ((button1.buttonClick == true) & (button1.buttonType == BUTTON_CLICK)) {
    return true;
  }
  else {
    return false;
  }
}

boolean buttonDouble () {
  if ((button1.buttonClick == true) & (button1.buttonType == BUTTON_DOUBLE)) {
    return true;
  }
  else {
    return false;
  }
}
//=======================================================================
void menuBRAGAorSS () {
  lcd.setCursor(0, 0); lcd.print("BRAGA-Long press");
  lcd.setCursor(0, 1); lcd.print("SS-Double click ");
}
//======================================================================
void menuRazgon () {
  lcd.setCursor(0, 0); lcd.print(F("       RAZGON       "));
  lcd.setCursor(0, 1); lcd.print(F("T: ")); lcd.print(temperature);
  lcd.print(F("   Tr: "));  lcd.print(tempRazg);
  lcd.setCursor(0, 2); lcd.print(F("CLICK: -   DOUBLE: +"));
  lcd.setCursor(0, 3); lcd.print(F("  LONG -  NA SEBYA  "));
}
//=======================================================================
void menuRazgonEnd () {
  lcd.setCursor(0, 0); lcd.print(F("  RAZGON ZAKONCHEN  "));
  lcd.setCursor(0, 1); lcd.print(F("                    "));
  lcd.setCursor(0, 2); lcd.print(F("                    "));
  lcd.setCursor(0, 3); lcd.print(F("  LONG -  NA SEBYA  "));
}
//=======================================================================
void menuNaSebya () {
  lcd.setCursor(0, 0); lcd.print(F("      NA SEBYA      "));
  lcd.setCursor(0, 1); lcd.print(F("T: ")); lcd.print(temperature); lcd.print(F(" TIMER: "));
  lcd.print(timerMin); lcd.print(":00");
  lcd.setCursor(0, 2); lcd.print(F("CLICK: -   DOUBLE: +"));
  lcd.setCursor(0, 3); lcd.print(F("   LONG -  GOLOVY   "));
}
//======================================================================
void menuNaSebyaEnd () {
  lcd.setCursor(0, 0); lcd.print(F(" NA SEBYA ZAKONCHEN "));
  lcd.setCursor(0, 1); lcd.print(F("                    "));
  lcd.setCursor(0, 2); lcd.print(F("                    "));
  lcd.setCursor(0, 3); lcd.print(F("   LONG -  GOLOVY   "));
}
//========================================================================
void menuShimG () {
  valveOn = valveOnHead; valveOff = valveOffHead;
  lcd.setCursor(0, 0); lcd.print(F("     SHIM GOLOVY    "));
  menuShim ();
}
//========================================================================
void menuShim () {
  lcd.setCursor(0, 1); lcd.print(F("       "));
  if (valveOn < 10) {
    lcd.print("0");
  }
  lcd.print(valveOn); lcd.print("/");
  if (valveOff < 10) {
    lcd.print("0");
  }
  lcd.print(valveOff); lcd.print(F("        "));
  lcd.setCursor(0, 2); lcd.print(F("CLICK: -   DOUBLE: +"));
  lcd.setCursor(0, 3); lcd.print(F("    LONG -  NEXT    "));
}
//=======================================================================
void menuShimGConfirmation () {
  lcd.setCursor(0, 0); lcd.print(F("     SHIM GOLOVY    "));
  menuShimConfirmation ();
}
//=======================================================================
void menuShimConfirmation () {
  lcd.setCursor(0, 1); lcd.print(F("       "));
  if (valveOn < 10) {
    lcd.print("0");
  }
  lcd.print(valveOn); lcd.print("/");
  if (valveOff < 10) {
    lcd.print("0");
  }
  lcd.print(valveOff); lcd.print(F("        "));
  menuYesNo ();
}
//=========================================================================
void menuYesNo () {
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - NO     "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - YES      "));
}
//==========================================================================
void menuGolovy () {
  lcd.setCursor(0, 0); lcd.print(F("       GOLOVY       "));
  lcd.setCursor(0, 1); lcd.print(F("T: ")); lcd.print(temperature); lcd.print(F("  "));
  lcd.print(F("SHIM:"));
  if (valveOn < 10) {
    lcd.print(F("0"));
  }
  lcd.print(valveOn); lcd.print(F("/"));
  if (valveOff < 10) {
    lcd.print(F("0"));
  }
  lcd.print(valveOff);
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - SHIM   "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - TELO     "));
}
//=============================================================================
void menuShimT () {
  lcd.setCursor(0, 0); lcd.print(F("      SHIM TELO     "));
  menuShim ();
}
//=============================================================================
void menuShimTConfirmation () {
  lcd.setCursor(0, 0); lcd.print(F("      SHIM TELO     "));
  menuShimConfirmation ();
}
//=============================================================================
void menuTelo () {
  lcd.setCursor(0, 0); lcd.print(F("        TELO        "));
  lcd.setCursor(0, 1); lcd.print(F("T: ")); lcd.print(temperature); lcd.print(F("  "));
  lcd.print(F("SHIM:"));
  if (valveOn < 10) {
    lcd.print("0");
  }
  lcd.print(valveOn); lcd.print("/");
  if (valveOff < 10) {
    lcd.print("0");
  }
  lcd.print(valveOff);
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - SHIM   "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - AUTO     "));
}
//=============================================================================
void menuAutoConfirmation () {
  lcd.setCursor(0, 0); lcd.print(F("        AUTO        "));
  lcd.setCursor(0, 1); lcd.print(F("                    "));
  menuYesNo ();
}
//=============================================================================
void menuAuto () {
  lcd.setCursor(0, 0);
  lcd.print(F("AUTO     SHIM: "));
  if (valveOn < 10) {
    lcd.print("0");
  }
  lcd.print(valveOn); lcd.print("/");
  if (valveOff < 10) {
    lcd.print("0");
  }
  lcd.print(valveOff);
  lcd.setCursor(0, 1);
  lcd.print(F("T:")); lcd.print(temperature);
  lcd.print(F("  Tauto:")); lcd.print(tempAuto);
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - SHIM   "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - Tauto    "));
}
//================================================================================
void menuStartStop () {
  lcd.setCursor(0, 0);
  lcd.print(F("     ")); lcd.print(F("Start/Stop")); lcd.print(F("     "));
  lcd.setCursor(0, 1);
  lcd.print(F("  ")); lcd.print(F("T:")); lcd.print(temperature); lcd.print(F("    "));
  lcd.print(F("00:00")); lcd.print(F("  "));
  lcd.setCursor(0, 2); lcd.print(F("    DOUBLE - SHIM   "));
  lcd.setCursor(0, 3); lcd.print(F("    LONG - Tauto    "));
}
//===================================================================================
void menuShimA () {
  lcd.setCursor(0, 0); lcd.print(F("      SHIM AUTO     "));
  menuShim ();
}
//===============================================================================
void menuShimAConfirmation () {
  lcd.setCursor(0, 0); lcd.print(F("      SHIM AUTO     "));
  menuShimConfirmation ();
}
//======================================================================================
void menuTauto () {
  lcd.setCursor(0, 0); lcd.print(F("        Tauto       "));
  lcd.setCursor(0, 1); lcd.print(F("        ")); lcd.print(temperature); lcd.print(F("        "));
  lcd.setCursor(0, 2); lcd.print(F("CLICK: -   DOUBLE: +"));
  lcd.setCursor(0, 3); lcd.print(F("    LONG -  NEXT    "));
}
//========================================================================================
void menuTautoConfirmation () {
  lcd.setCursor(0, 0); lcd.print(F("        Tauto       "));
  lcd.setCursor(0, 1); lcd.print(F("        "));
  lcd.print(temperature); lcd.print(F("        "));
  menuYesNo ();
}
//===============================================================
void menuOtborEnd () {
  lcd.setCursor(0, 0); lcd.print(F("                    "));
  lcd.setCursor(0, 1); lcd.print(F("     "));
  lcd.print(F("OTBOR STOP")); lcd.print(F("     "));
  lcd.setCursor(0, 2); lcd.print(F("                    "));
  lcd.setCursor(0, 3); lcd.print(F("                    "));
}
//===============================================================
void Shim(byte paramMin, byte paramMax, byte paramJob) {
  button1.scanState();
  if ((buttonOne ()) & (valveOn > paramMin)) {
    valveOn-- ; valveOff++ ;
  }
  if ((buttonDouble ()) & (valveOn < paramMax)) {
    valveOn++ ; valveOff-- ;
  }
  if (buttonLong ()) {
    if (paramJob == ShimGConfirmation) {
      menuShimGConfirmation (); jobState = ShimGConfirmation;
    }
    if (paramJob == ShimTConfirmation) {
      menuShimTConfirmation (); jobState = ShimTConfirmation;
    }
    if (paramJob == ShimAConfirmation) {
      menuShimAConfirmation (); jobState = ShimAConfirmation;
    }
    if (paramJob == ShimSSConfirmation) {
      menuShimAConfirmation (); jobState = ShimAConfirmation;
    }
  }

  lcd.setCursor(7, 1);
  if (valveOn < 10) {
    lcd.print(F("0"));
  }
  lcd.print(valveOn);

  lcd.setCursor(10, 1);
  if (valveOff < 10) {
    lcd.print(F("0"));
  }
  lcd.print(valveOff);

  button1.buttonClick = false;
}
//=================================================================
void ShimConfirmation (byte paramJobYes, byte paramJobNo) {
  button1.scanState();
  if (buttonLong ()) {
    countOn = valveOn; countOff = valveOff;
    modeValve = HIGH; digitalWrite(VALVE_PIN, HIGH);
    jobState = paramJobYes;
    if (jobState == Golovy) {
      menuGolovy();
    }
    if (jobState == Telo) {
      menuTelo();
    }
    if (jobState == Auto) {
      menuAuto();
    }
    if (jobState == StartStop) {
      menuStartStop();
    }
  }
  if (buttonDouble ()) {
    valveOn = valveOnTmp; valveOff = valveOffTmp;
    jobState = paramJobNo;
    if (jobState == ShimG) {
      menuShimG();
    }
    if (jobState == ShimT) {
      menuShimT();
    }
    if (jobState == ShimA) {
      menuShimA();
    }
    if (jobState == ShimSS) {
      menuShimA();
    }
  }
  button1.buttonClick = false;
}
